/**
 * Modulo: sieteYmedia.c
 *
 * Descripcion: programa para jugar al juego de cartas "Las siete y media".
 *
 * Autor: Ivan Cantador, ivan.cantador@uam.es
 *
 * Version: 1.1
 * Fecha: 16/10/2017
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// ================================
// Definicion de valores constantes
// ================================

#define AS 		1
#define DOS		2
#define TRES	3
#define CUATRO	4
#define CINCO	5
#define SEIS	6
#define SIETE 	7
#define SOTA 	0.5
#define CABALLO	0.5
#define REY 	0.5

#define MAX_JUGADORES	4

// =======================
// Definicion de funciones
// =======================

int aleatorio(int a, int b) {
	return (int) (a + (double) rand() / (RAND_MAX-1) * (b - a + 1));
}

// =================
// Funcion principal
// =================

int main() {
	// Baraja
	double cartas[40] = {AS, DOS, TRES, CUATRO, CINCO, SEIS, SIETE, SOTA, CABALLO, REY,
			AS, DOS, TRES, CUATRO, CINCO, SEIS, SIETE, SOTA, CABALLO, REY,
			AS, DOS, TRES, CUATRO, CINCO, SEIS, SIETE, SOTA, CABALLO, REY,
			AS, DOS, TRES, CUATRO, CINCO, SEIS, SIETE, SOTA, CABALLO, REY};
	// Puntos de los jugadores
	double puntos[MAX_JUGADORES];
	// Otras variables
	int opcion, numJugadores, j, c, c1, c2;
	double mejorPuntuacion, t;

	// Inicializamos el generador de numeros aleatorios
	srand((unsigned) time(NULL));

	// Barajamos las cartas, intercambiando pares de cartas varias veces
	for ( j = 0; j < 100; j++ ) {
		c1 = aleatorio(0, 39);
		c2 = aleatorio(0, 39);

		t = cartas[c1];
		cartas[c1] = cartas[c2];
		cartas[c2] = t;
	}

	// Solicitamos el numero de jugadores en la partida
	numJugadores = 0;
	while ( numJugadores == 0 ) {
		printf("Cuantos jugadores (maximo %d)? ", MAX_JUGADORES);
		scanf("%d", &opcion);
		if ( opcion > 0 && opcion <= MAX_JUGADORES ) {
			numJugadores = opcion;
		}
	}

	// Inicializamos los puntos de los jugadores
	for ( j=0; j<numJugadores; j++ ) {
		puntos[j] = 0.0;
	}

	// Jugamos la partida...
	c = 0;
	for ( j=0; j<numJugadores; j++ ) {
		// Damos una primera carta al jugador
		puntos[j] += cartas[c];
		c++;

		printf("\nJugador %d ==========\n", j+1);
		printf("Tu puntuacion parcial es %.1lf.\n", puntos[j]);

		// De forma iterativa preguntamos al jugador si quiere mas cartas
		while ( puntos[j] < 7.5 ) {
			printf("Quieres carta (s/n)? ");
			scanf("\n%c", (char *) &opcion);
			if ( opcion == 's' || opcion == 'S' ) {
				puntos[j] += cartas[c];
				c++;

				printf("Tu puntuacion parcial es %.1lf.\n", puntos[j]);

				continue;
			}
			printf("Te has plantado!\n");
			break;
		}
		printf("Te quedas con %.1f puntos!\n", puntos[j]);
	}

	// Visualizamos por pantalla el/los ganador(es)
	mejorPuntuacion = 0;
	for ( j=0; j<numJugadores; j++ ) {
		if ( puntos[j] <= 7.5 && puntos[j] > mejorPuntuacion ) {
			mejorPuntuacion = puntos[j];
		}
	}

	printf("\nLa mejor puntuacion es %.1lf.\n", mejorPuntuacion);
	printf("El/los jugador(es) ganador(es) es/son:\n");
	for ( j=0; j<numJugadores; j++ ) {
		if ( puntos[j] == mejorPuntuacion ) {
			printf("\tJugador %d.\n", j+1);
		}
	}

	return 0;
}
